# topicmaster.tcl v1.o7g by domsen <domsen@domsen.org> (c)2oo1
#
# comments? bugs? ideas? plz mail me :)
#
# topicmaster is an advanced topic management tcl including the following
# functions:
# 
# -if the topic gets empty due to netsplit the last topic will be set again
# -returns infos about the currently set topic: nickname, host, date, time and so on
# -saves every topic to a historyfile
# -sets random topics from the historyfiles
# -possibility to recieve the historyfile via dcc
# -possibility to get a short topichistory
# -possibility to get the last topic number x
# -owners can reset the historyfiles
# -all commands available via msg and pub
# -basic html output
# -different settings for each chan
# -online help
#
# history:
# v1.07g - unmatched open quote bug fixed
# v1.07f - basic html output, extra space bug fixed
# v1.05 - topiccounter added
# v1.0 - 1st final version
#
# config #

set topictrigger ".topic"
set topichistory ".topichistory"
set lasttopics ".lasttopic"
set erasehistory "erasehistory"
set randomt ".random"
set gethistory ".gethistory"
set topichelp ".topichelp"
set topicversion "v1.o7g"
set htmloutput "yes"
set topicchans ""

# dont change anything below #

bind topc - * savetopic
bind topc - * topicleer
bind pub - $topictrigger pub_topic
bind msg - $topictrigger msg_topic

bind msg - $topichistory msg_history
bind pub - $topichistory pub_history

bind msg - $lasttopics msg_lasttopics
bind pub - $lasttopics pub_lasttopics

bind dcc n $erasehistory msg_erasehistory

bind pub - $randomt pub_randomtopic
bind msg - $randomt msg_randomtopic

bind pub - $gethistory pub_gethistory
bind msg - $gethistory msg_gethistory

bind msg - $topichelp msg_topichelp
bind pub - $topichelp pub_topichelp


proc msg_topichelp {nick uhost hand arg} {
global topicversion topictrigger topichistory lasttopics erasehistory randomt gethistory topichelp
putserv "privmsg $nick :topicmaster $topicversion by domsen <domsen@domsen.org>"
putserv "privmsg $nick :-"
putserv "privmsg $nick :the following pub commands are available:"
putserv "privmsg $nick :$topictrigger <?channel?> :returns infos about the topic"
putserv "privmsg $nick :$topichistory <?channel?> :returns the last 3 topics" 
putserv "privmsg $nick :$lasttopics <number> :returns the last topic number <number>"
putserv "privmsg $nick :$randomt <?channel?> :sets a random old topic"
putserv "privmsg $nick :$gethistory <?channel?> :sends the channels historyfile"
putserv "privmsg $nick :$topichelp :shows this help"
putserv "privmsg $nick :-"
putserv "privmsg $nick :the following msg commands are available:"
putserv "privmsg $nick :$topictrigger <channel> :returns infos about the topic"
putserv "privmsg $nick :$topichistory <channel> :returns the last 3 topics" 
putserv "privmsg $nick :$lasttopics <channel> <number> :returns the last topic number <number>"
putserv "privmsg $nick :$randomt <channel> :sets a random old topic"
putserv "privmsg $nick :$gethistory <channel> :sends the channels historyfile"
putserv "privmsg $nick :$topichelp :shows this help"
putserv "privmsg $nick :-"
putserv "privmsg $nick :the following pub commands are available:"
putserv "privmsg $nick :$erasehistory <channel> :deletes the history file (owners only)"
}

proc pub_topichelp {nick uhost hand chan arg} {
global topicversion topictrigger topichistory lasttopics erasehistory randomt gethistory topichelp topicchans
if {([lsearch -exact [string tolower $topicchans] [string tolower $chan]] != -1) || ($topicchans == "")} {
putserv "privmsg $nick :topicmaster $topicversion by domsen <domsen@domsen.org>"
putserv "privmsg $nick :-"
putserv "privmsg $nick :the following pub commands are available:"
putserv "privmsg $nick :$topictrigger <?channel?> :returns infos about the topic"
putserv "privmsg $nick :$topichistory <?channel?> :returns the last 3 topics" 
putserv "privmsg $nick :$lasttopics <number> :returns the last topic number <number>"
putserv "privmsg $nick :$randomt <?channel?> :sets a random old topic"
putserv "privmsg $nick :$gethistory <?channel?> :sends the channels historyfile"
putserv "privmsg $nick :$topichelp :shows this help"
putserv "privmsg $nick :-"
putserv "privmsg $nick :the following msg commands are available:"
putserv "privmsg $nick :$topictrigger <channel> :returns infos about the topic"
putserv "privmsg $nick :$topichistory <channel> :returns the last 3 topics" 
putserv "privmsg $nick :$lasttopics <channel> <number> :returns the last topic number <number>"
putserv "privmsg $nick :$randomt <channel> :sets a random old topic"
putserv "privmsg $nick :$gethistory <channel> :sends the channels historyfile"
putserv "privmsg $nick :$topichelp :shows this help"
putserv "privmsg $nick :-"
putserv "privmsg $nick :the following dcc commands are available:"
putserv "privmsg $nick :$erasehistory <channel> :deletes the history file (owners only)"
}}


proc msg_gethistory {nick uhost hand arg} {
global gethistory
if {$arg == ""} {
 putserv "privmsg $nick :usage: $gethistory <channel>"
 return 0}
if { $arg != ""} {
set historyfile "history.$args"
if {[file exists $historyfile]} {
switch -- [dccsend $historyfile $nick] {
0 { putserv "notice $nick :sending..." }
1 { putserv "notice $nick :error" }
2 { putserv "notice $nick :error" }
3 { putserv "notice $nick :error" }
4 { putserv "notice $nick :error" }
return 0
}}}}




proc pub_gethistory {nick uhost hand chan arg} {
global topicchans
if {([lsearch -exact [string tolower $topicchans] [string tolower $chan]] != -1) || ($topicchans == "")} {
if { $arg == ""} {set historyfile history.$chan }
if { $arg != ""} {set historyfile history.$arg }
if {[file exists $historyfile]} {
switch -- [dccsend $historyfile $nick] {
0 { putserv "notice $nick :sending..." }
1 { putserv "notice $nick :error" }
2 { putserv "notice $nick :error" }
3 { putserv "notice $nick :error" }
4 { putserv "notice $nick :error" }
return 0
}}}}

proc makeheader {channel} {

set htmlfile "$channel.html"
set bodycolor {<body bgcolor="#FFFFFF" text="#000000">}
set tablestuff {<table width="75%" border="0">}
set dateichen [open $htmlfile w+]
puts $dateichen "<html>"
puts $dateichen "<head>"
puts $dateichen "<title>$channel topic history</title>"
puts $dateichen "</head>"
puts $dateichen "$bodycolor"
puts $dateichen "$tablestuff"
close $dateichen
}


proc makebottom {channel} {

set htmlfile "$channel.html"
set dateichen [open $htmlfile a]
puts $dateichen "</table>"
puts $dateichen "</body>"
puts $dateichen "</html>"
close $dateichen
}





proc dohtml {channel} {
global pubhtmltdir
file delete "$channel.html"
makeheader $channel
set historyfile "history.$channel"
set dateichen [open "$historyfile" r]
set htmlfile "$channel.html"
set dateichen2 [open "$htmlfile" a]
while {![eof $dateichen]} {
set zeile [gets $dateichen]
if {$zeile != ""} {
set zeile [split $zeile]
set nickname [lindex $zeile 0]
set channel [lindex $zeile 1]
set hostmask [lindex $zeile 2]
set weekday [lindex $zeile 3]
set month [lindex $zeile 4]
set day [lindex $zeile 5]
set daytime [lindex $zeile 6]
set year [lindex $zeile 7]
set topic [join [lrange $zeile 8 [llength $zeile]]]
puts $dateichen2 "<tr>"
puts $dateichen2 {<td width="30%">}
puts $dateichen2 "$daytime $weekday $day $month $year </td>"
puts $dateichen2 {<td width="70%">}
puts $dateichen2 {<div align="right">}
puts $dateichen2 "$nickname $hostmask </div>"
puts $dateichen2 "</td>"
puts $dateichen2 "</tr>"
puts $dateichen2 "<tr>"
puts $dateichen2 {<td width="30%">&nbsp;</td>}
puts $dateichen2 {<td width="70%">}
puts $dateichen2 "$topic </td>"
puts $dateichen2 "</tr>"
 }
}


close $dateichen
close $dateichen2
makebottom $channel
} 


proc countlines {arg} {
set counter 0
set channel [lindex $arg 0]
set historyfile "history.$channel"
set dateichen [open "$historyfile" r]
while {![eof $dateichen]} {
if {[gets $dateichen] != ""} {
incr counter }
}
close $dateichen
return $counter
}


proc msg_randomtopic {nick uhost hand arg} {
global randomt topicchans
if {$arg == ""} {
 putserv "privmsg $nick :usage: $randomt <channel>"
 return 0}
if { $arg != ""} {
if {([lsearch -exact [string tolower $topicchans] [string tolower $arg]] != -1) || ($topicchans == "")} {
set gesamtlines 0
set historyfile "history.$arg"
if {[file exists $historyfile]} {
set dateichen [open "$historyfile" r]
while {![eof $dateichen]} {
if {[gets $dateichen] != ""} {
incr gesamtlines }
}
close $dateichen
set nummer [rand $gesamtlines]
set counter 0
set dateichen2 [open "$historyfile" r]
while {![eof $dateichen2]} {
set zeile [gets $dateichen2]
if {$counter == $nummer} {
set zeile [split $zeile]
set nickname [lindex $zeile 0]
set channel [lindex $zeile 1]
set hostmask [lindex $zeile 2]
set weekday [lindex $zeile 3]
set month [lindex $zeile 4]
set day [lindex $zeile 5]
set daytime [lindex $zeile 6]
set year [lindex $zeile 7]
set topic [join [lrange $zeile 8 [llength $zeile]]]
putserv "topic $arg :$topic"
putserv "notice $nick :($counter/$gesamtlines) set by $nickname ($hostmask) $daytime $weekday $day $month $year"}
incr counter}
close $dateichen2
}}}}


proc pub_randomtopic {nick uhost hand chan arg} {
global topicchans
if {([lsearch -exact [string tolower $topicchans] [string tolower $chan]] != -1) || ($topicchans == "")} {
if { $arg == ""} {set historyfile history.$chan
set channel $chan }
if { $arg != ""} {set historyfile history.$arg
set channel $arg }
set gesamtlines 0
if {[file exists $historyfile]} {
set dateichen [open "$historyfile" r]
while {![eof $dateichen]} {
if {[gets $dateichen] != ""} {
incr gesamtlines }
}
close $dateichen
set nummer [rand $gesamtlines]
set counter 0
set dateichen2 [open "$historyfile" r]
while {![eof $dateichen2]} {
set zeile [gets $dateichen2]
if {$counter == $nummer} {
set zeile [split $zeile]
set nickname [lindex $zeile 0]
set channel [lindex $zeile 1]
set hostmask [lindex $zeile 2]
set weekday [lindex $zeile 3]
set month [lindex $zeile 4]
set day [lindex $zeile 5]
set daytime [lindex $zeile 6]
set year [lindex $zeile 7]
set topic [join [lrange $zeile 8 [llength $zeile]]]
putserv "topic $channel :$topic"
putserv "notice $nick :topic ($counter/$gesamtlines) set by $nickname ($hostmask)  $daytime $weekday $day $month $year"}
incr counter}
close $dateichen2
}}}




proc msg_erasehistory {hand idx arg} {
global erasehistory
if {$arg == ""} {
putdcc $idx "usage: $erasehistory <channel>"
 return 0}
if {$arg != ""} {
set historyfile "history.$arg"
if {[file exists $historyfile]} {
set olerocks [countlines $arg]
file delete "$historyfile"
putdcc $idx "$historyfile deleted, $olerocks topic(s) lost"
}}}


proc msg_lasttopics {nick uhost hand arg} {
global lasttopics
if {$arg == ""} {
 putserv "privmsg $nick :usage: $lasttopics <channel> <topicnumber>"
 return 0}
if { $arg != ""} {
set channel [lindex $arg 0]
set pointer [lindex $arg 1]
set counter "1"
if {$pointer > "0"} {
set historyfile "history.$channel"
if {[file exists $historyfile]} {
set dateichen [open "$historyfile" r]
while {![eof $dateichen]} {
set zeile [gets $dateichen]
if {$counter == $pointer} {
set zeile [split $zeile]
set nickname [lindex $zeile 0]
set channel [lindex $zeile 1]
set hostmask [lindex $zeile 2]
set weekday [lindex $zeile 3]
set month [lindex $zeile 4]
set day [lindex $zeile 5]
set daytime [lindex $zeile 6]
set year [lindex $zeile 7]
set topic [join [lrange $zeile 8 [llength $zeile]]]
set olerocks [countlines $channel]
putserv "privmsg $nick :last topic number $pointer of $olerocks: '$topic' was set by $nickname ($hostmask) at $daytime $weekday $day $month $year"
return 0
}
incr counter
}}}}}

proc pub_lasttopics {nick uhost hand chan arg} {
global lasttopics topicchans
if {([lsearch -exact [string tolower $topicchans] [string tolower $chan]] != -1) || ($topicchans == "")} {
if {$arg == ""} {
 putserv "privmsg $nick :usage: $lasttopics <topicnumber>"
 return 0}
if { $arg != ""} {
set pointer [lindex $arg 0]
set counter "1"
if {$pointer > "0"} {
set historyfile "history.$chan"
if {[file exists $historyfile]} {
set dateichen [open "$historyfile" r]
while {![eof $dateichen]} {
set zeile [gets $dateichen]
if {$counter == $pointer} {
set zeile [split $zeile]
set nickname [lindex $zeile 0]
set channel [lindex $zeile 1]
set hostmask [lindex $zeile 2]
set weekday [lindex $zeile 3]
set month [lindex $zeile 4]
set day [lindex $zeile 5]
set daytime [lindex $zeile 6]
set year [lindex $zeile 7]
set topic [join [lrange $zeile 8 [llength $zeile]]]
set olerocks [countlines $chan]
putserv "notice $nick :last topic number $pointer of $olerocks: '$topic' was set by $nickname ($hostmask) at $daytime $weekday $day $month $year"
return 0
}
incr counter
}}}}}}



proc msg_history {nick uhost hand arg} {
global topichistory
if {$arg == ""} {
 putserv "privmsg $nick :usage: $topichistory <channel>"
 return 0}
if { $arg != ""} {
set historyfile history.$arg
if {[file exists $historyfile]} { 
set dateichen [open "$historyfile" r]
set counter "0"
while {![eof $dateichen]} {
set zeile [gets $dateichen]
set zeile [split $zeile]
set nickname [lindex $zeile 0]
set channel [lindex $zeile 1]
set hostmask [lindex $zeile 2]
set weekday [lindex $zeile 3]
set month [lindex $zeile 4]
set day [lindex $zeile 5]
set daytime [lindex $zeile 6]
set year [lindex $zeile 7]
set topic [join [lrange $zeile 8 [llength $zeile]]]
if {$counter != "3"} {
if {$day != ""} {
set olerocks [countlines $arg]
putserv "privmsg $nick :($olerocks topics available) the topic '$topic' was set by $nickname ($hostmask) at $daytime $weekday $day $month $year"}
incr counter
}}
close $dateichen
}}}



proc pub_history {nick uhost hand chan arg} {
global topicchans
if {([lsearch -exact [string tolower $topicchans] [string tolower $chan]] != -1) || ($topicchans == "")} {
if { $arg == ""} {
set historyfile "history.$chan"
set kanal $chan
}
if { $arg != ""} {
set historyfile "history.$arg"
set kanal $arg }
if {[file exists $historyfile]} {
set dateichen [open "$historyfile" r]
set counter "0"
while {![eof $dateichen]} {
set zeile [gets $dateichen]
set zeile [split $zeile]
set nickname [lindex $zeile 0]
set channel [lindex $zeile 1]
set hostmask [lindex $zeile 2]
set weekday [lindex $zeile 3]
set month [lindex $zeile 4]
set day [lindex $zeile 5]
set daytime [lindex $zeile 6]
set year [lindex $zeile 7]
set topic [join [lrange $zeile 8 [llength $zeile]]]
if {$counter != "3"} {
if {$day != ""} {
set olerocks [countlines $kanal]
putserv "notice $nick :($olerocks topics available) the topic '$topic' was set by $nickname ($hostmask) at $daytime $weekday $day $month $year"}
incr counter
}}
close $dateichen
}}}


proc savetopic {nick uhost hand chan topic} {
global topicchans
if {([lsearch -exact [string tolower $topicchans] [string tolower $chan]] != -1) || ($topicchans == "")} {
global botnick htmloutput
if {($nick != "*") && ($uhost != "*") && ($topic != "") } {

set topicfile topic.$chan
set historyfile history.$chan
set tempfile temp.$chan
if {[file exists $topicfile]} { 
set dateichen [open "$topicfile" r]
set oldtopic [gets $dateichen]
close $dateichen
} else {set oldtopic ""}

set dateichen2 [open "$tempfile" w+]
puts $dateichen2 "$oldtopic"
close $dateichen2


if {[file exists $historyfile]} { 
set dateichen3 [open "$historyfile" r]
} else { 
set dateichen3 [open "$historyfile" w+]
close $dateichen3
set dateichen3 [open "$historyfile" r]
}
set dateichen4 [open "$tempfile" a]
while {![eof $dateichen3]} {
set zeile [gets $dateichen3]
puts $dateichen4 "$zeile"
}
close $dateichen3
close $dateichen4
set dateichen5 [open "$historyfile" w]
set dateichen6 [open "$tempfile" r]
while {![eof $dateichen6]} {
set zeile [gets $dateichen6]
puts $dateichen5 "$zeile"
}
close $dateichen5
close $dateichen6
set time [ctime [unixtime]]
set dateichen7 [open "$topicfile" w]
puts $dateichen7 "$nick $chan $uhost $time $topic"
close $dateichen7
if {$htmloutput == "yes"} {
dohtml $chan
}
}}}


proc pub_topic {nick uhost hand chan arg} {
global topicchans
if {([lsearch -exact [string tolower $topicchans] [string tolower $chan]] != -1) || ($topicchans == "")} {
if { $arg == ""} {set topicfile topic.$chan }
if { $arg != ""} {set topicfile topic.$arg }
if {[file exists $topicfile]} { 
set dateichen [open "$topicfile" r]
set temp [gets $dateichen]
set temp [split $temp]
close $dateichen
set nickname [lindex $temp 0]
set channel [lindex $temp 1]
set hostmask [lindex $temp 2]
set weekday [lindex $temp 3]
set month [lindex $temp 4]
set day [lindex $temp 5]
set daytime [lindex $temp 6]
set year [lindex $temp 7]
set topic [join [lrange $temp 8 [llength $temp]]]
putserv "privmsg $chan :the topic '$topic' was set by $nickname ($hostmask) at $daytime $weekday $day $month $year"
}}}


proc msg_topic {nick uhost hand arg} {
global topictrigger
if { $arg == ""} {
 putserv "privmsg $nick :usage $topictrigger <channel>"
 return 0}
if { $arg != ""} {
set topicfile topic.$arg
if {[file exists $topicfile]} { 
set dateichen [open "$topicfile" r]
set temp [gets $dateichen]
set temp [split $temp]
close $dateichen
set nickname [lindex $temp 0]
set channel [lindex $temp 1]
set hostmask [lindex $temp 2]
set weekday [lindex $temp 3]
set month [lindex $temp 4]
set day [lindex $temp 5]
set daytime [lindex $temp 6]
set year [lindex $temp 7]
set topic [join [lrange $temp 8 [llength $temp]]]
putserv "privmsg $nick :the topic '$topic' was set by $nickname ($hostmask) at $daytime $weekday $day $month $year"}
}}


proc topicleer { nick uhost hand chan topic } {
if {$topic == ""} {
set topicfile "topic.$chan"
if {[file exists $topicfile]} { 
set dateichen [open "$topicfile" r]
set temp [gets $dateichen]
set temp [split $temp]
close $dateichen
set newtopic [join [lrange $temp 8 [llength $temp]]]
putserv "topic $chan :$newtopic"
}}}

putlog "*** topicmaster.tcl $topicversion by domsen succesfully loaded"

#[EOF]